from colors import red


class InvalidDataType(Exception):
    def __init__(self, message=f"{red('[ERROR]')} Invalid data type, try '-' for stdin, valid URL or path on your file system."):
        self.message = message
        super().__init__(self.message)


class InvalidInput(Exception):
    def __init__(self, message=f"{red('[ERROR]')} Invalid input, stdin can be used for data or template but not both."):
        self.message = message
        super().__init__(self.message)



import json
import os
import requests
import validators
import yaml
import sys

from .exceptions import InvalidDataType
from colors import red, green
from jinja2 import Template as Jinja2_template, exceptions


def print_verbose(message):
    if message['verbose']:
        separator = green(f'{"-" * 10}')
        print(separator, message['title'], separator)
        print(green(message['content']), "\n")


def load_yaml(data):
    return yaml.load(data, Loader=yaml.FullLoader)


def input_handler(data):
    if data == '-':
        return sys.stdin.read()
    elif validators.url(data):
        r = requests.get(data)
        if r.status_code < 400:
            return r.text
    elif os.path.exists(data):
        with open(data, 'r') as input_data_file:
            return input_data_file.read()
    else:
        raise InvalidDataType()


class Base:
    def __init__(self, data, verbose):
        self.data = data
        self.verbose = verbose


class Data(Base):
    def __init__(self, data, verbose):
        Base.__init__(self, data, verbose)

    def get_data(self):
        try:
            ih = input_handler(self.data)
        except InvalidDataType as err:
            print(err.message)
        else:
            self.data = load_yaml(ih)
            print_verbose({'title': '[Data]', 'content': json.dumps(
                self.data, indent=2), 'verbose': self.verbose})
            return self.data


class Template(Base):
    def __init__(self, template, verbose, data, no_trim_blocks, no_lstrip_blocks):
        Base.__init__(self, data, verbose)
        self.template = template
        self.no_trim_blocks = no_trim_blocks
        self.no_lstrip_blocks = no_lstrip_blocks

    def __render(self):
        try:
            if self.data is not None:
                return self.template.render(self.data)
            else:
                return self.template.render()
        except exceptions.UndefinedError as err:
            print(red('[ERROR]'), err.message)

    def get_rendered_template(self):
        try:
            ih = input_handler(self.template)
        except InvalidDataType as err:
            print(err.message)
        else:
            print_verbose(
                {'title': '[Template]', 'content': ih, 'verbose': self.verbose})
            self.template = Jinja2_template(
                ih, trim_blocks=self.no_trim_blocks, lstrip_blocks=self.no_lstrip_blocks)
            return self.__render()



import click

from .handlers import Data, Template
from .exceptions import InvalidInput

@click.group()
def main():
    pass


@main.command()
@click.option('--data', '-d', help="PATH to YAML or JSON data file, URL or '-' for stdin")
@click.option('--template', '-t', help="PATH to any file that uses Jinja, URL or '-' for stdin")
@click.option('--verbose', '-v', is_flag=True)
@click.option('--no-trim-blocks', '-tb', default=True, is_flag=True, help='Disable trim blocks')
@click.option('--no-lstrip-blocks', '-lb', default=True, is_flag=True, help='Disable lstrip blocks')
@click.option('--output', '-o', type=click.Path(), help='PATH for output, stdout by default')
def render(data, template, verbose, no_trim_blocks, no_lstrip_blocks, output):
    if data == '-' and template == '-':
        raise InvalidInput()
    
    if data is not None:
        data = Data(data, verbose).get_data()

    if template is not None:
        out = Template(template, verbose, data, no_trim_blocks, no_lstrip_blocks).get_rendered_template()
        if out is not None:
            if output is not None:
                with open(output, 'w+') as output_file:
                    output_file.write(out)
            else:
                print(out)
